@@ -1,15 +1,10 @@
 """
-Classes for generating SAM and BAM files and records for testing
-----------------------------------------------------------------
+# Classes for generating SAM and BAM files and records for testing
 
 This module contains utility classes for the generation of SAM and BAM files and
 alignment records, for use in testing.
-
-The module contains the following public classes:
-
-    - :class:`~fgpyo.sam.builder.SamBuilder` -- A builder class that allows the accumulation
-        of alignment records and access as a list and writing to file.
 """
+
 from array import array
 from pathlib import Path
 from random import Random
@@ -21,6 +16,7 @@ from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Tuple
+from typing import cast
 
 import pysam
 from pysam import AlignedSegment
@@ -39,13 +35,14 @@ class SamBuilder:
     A builder is constructed with a handful of defaults including lengths for generated R1s
     and R2s, the default base quality score to use, a sequence dictionary and a single read group.
 
-    Records are then added using the :func:`~fgpyo.builder.SamBuilder.add_pair` method.
-    Once accumulated the records can be accessed in the order in which they were created through
-    the :func:`~fgpyo.builder.SamBuilder.to_unsorted_list` function, or in a list sorted
-    by coordinate order via :func:`~fgpyo.builder.SamBuilder.to_sorted_list`.  The latter
-    creates a temporary file to do the sorting and is somewhat slower as a result.  Lastly, the
-    records can be written to a temporary file using
-    :func:`~fgpyo.builder.SamBuilder.to_path`.
+    Records are then added using the [`add_pair()`][fgpyo.sam.builder.SamBuilder.add_pair]
+    method.  Once accumulated the records can be accessed in the order in which they were created
+    through the [`to_unsorted_list()`][fgpyo.sam.builder.SamBuilder.to_unsorted_list]
+    function, or in a list sorted by coordinate order via
+    [`to_sorted_list()`][fgpyo.sam.builder.SamBuilder.to_sorted_list].  The latter creates
+    a temporary file to do the sorting and is somewhat slower as a result.  Lastly, the records can
+    be written to a temporary file using
+    [`to_path()`][fgpyo.sam.builder.SamBuilder.to_path].
     """
 
     # The default read one length
@@ -117,7 +114,7 @@ class SamBuilder:
             sd: a sequence dictionary as a list of dicts; defaults to calling default_sd() if None
             rg: a single read group as a dict; defaults to calling default_sd() if None
             extra_header: a dictionary of extra values to add to the header, None otherwise.  See
-                          `::class::~pysam.AlignmentHeader` for more details.
+                          `pysam.AlignmentHeader` for more details.
             seed: a seed value for random number/string generation
             sort_order: Order to sort records when writing to file, or output of to_sorted_list()
         """
@@ -153,7 +150,7 @@ class SamBuilder:
 
     def _bases(self, length: int) -> str:
         """Returns a random string of bases of the length requested."""
-        return "".join(self._random.choices("ACGT", k=length))  # type: ignore
+        return "".join(self._random.choices("ACGT", k=length))
 
     def _new_rec(
         self,
@@ -267,79 +264,19 @@ class SamBuilder:
         if not rec.is_unmapped:
             rec.cigarstring = cigar if cigar else f"{length}M"
 
-    def _set_mate_info(self, r1: pysam.AlignedSegment, r2: pysam.AlignedSegment) -> None:
-        """Sets the mate information on a pair of sam records.
-
-        Handles cases where both reads are mapped, one of the two reads is unmapped or both reads
-        are unmapped.
-
-        Args:
-            r1: the first read in the pair
-            r2: the sceond read in the pair
-        """
-        for rec in r1, r2:
-            rec.template_length = 0
-            rec.is_proper_pair = False
-
-        if r1.is_unmapped and r2.is_unmapped:
-            # If they're both unmapped just clean the records up
-            for rec, other in [(r1, r2), (r2, r1)]:
-                rec.reference_id = sam.NO_REF_INDEX
-                rec.next_reference_id = sam.NO_REF_INDEX
-                rec.reference_start = sam.NO_REF_POS
-                rec.next_reference_start = sam.NO_REF_POS
-                rec.is_unmapped = True
-                rec.mate_is_unmapped = True
-                rec.is_proper_pair = False
-                rec.mate_is_reverse = other.is_reverse
-
-        elif r1.is_unmapped or r2.is_unmapped:
-            # If only one is mapped/unmapped copy over the relevant stuff
-            (m, u) = (r1, r2) if r2.is_unmapped else (r2, r1)
-            u.reference_id = m.reference_id
-            u.reference_start = m.reference_start
-            u.next_reference_id = m.reference_id
-            u.next_reference_start = m.reference_start
-            u.mate_is_reverse = m.is_reverse
-            u.mate_is_unmapped = False
-            u.set_tag("MC", m.cigarstring)
-
-            m.next_reference_id = u.reference_id
-            m.next_reference_start = u.reference_start
-            m.mate_is_reverse = u.is_reverse
-            m.mate_is_unmapped = True
-
-        else:
-            # Else they are both mapped
-            for rec, other in [(r1, r2), (r2, r1)]:
-                rec.next_reference_id = other.reference_id
-                rec.next_reference_start = other.reference_start
-                rec.mate_is_reverse = other.is_reverse
-                rec.mate_is_unmapped = False
-                rec.set_tag("MC", other.cigarstring)
-
-            if r1.reference_id == r2.reference_id:
-                r1p = r1.reference_end if r1.is_reverse else r1.reference_start
-                r2p = r2.reference_end if r2.is_reverse else r2.reference_start
-                r1.template_length = r2p - r1p
-                r2.template_length = r1p - r2p
-
-                # Arbitrarily set proper pair if the we have an FR pair with isize <= 1000
-                if r1.is_reverse != r2.is_reverse and abs(r1.template_length) <= 1000:
-                    fpos, rpos = (r2p, r1p) if r1.is_reverse else (r1p, r2p)
-                    if fpos < rpos:
-                        r1.is_proper_pair = True
-                        r2.is_proper_pair = True
-
     def rg(self) -> Dict[str, Any]:
         """Returns the single read group that is defined in the header."""
-        rgs = self._header["RG"]
+        # The `RG` field contains a list of read group mappings
+        # e.g. `[{"ID": "rg1", "PL": "ILLUMINA"}]`
+        rgs = cast(List[Dict[str, Any]], self._header["RG"])
         assert len(rgs) == 1, "Header did not contain exactly one read group!"
         return rgs[0]
 
     def rg_id(self) -> str:
         """Returns the ID of the single read group that is defined in the header."""
-        return self.rg()["ID"]
+        # The read group mapping has mixed types of values (e.g. "PI" is numeric), but the "ID"
+        # field is always a string.
+        return cast(str, self.rg()["ID"])
 
     def add_pair(
         self,
@@ -443,8 +380,16 @@ class SamBuilder:
             raise ValueError("Cannot use chrom in combination with chrom1 or chrom2")
 
         chrom = sam.NO_REF_NAME if chrom is None else chrom
-        chrom1 = next(c for c in (chrom1, chrom) if c is not None)
-        chrom2 = next(c for c in (chrom2, chrom) if c is not None)
+
+        if start1 != sam.NO_REF_POS:
+            chrom1 = next(c for c in (chrom1, chrom) if c is not None)
+        else:
+            chrom1 = sam.NO_REF_NAME
+
+        if start2 != sam.NO_REF_POS:
+            chrom2 = next(c for c in (chrom2, chrom) if c is not None)
+        else:
+            chrom2 = sam.NO_REF_NAME
 
         if chrom1 == sam.NO_REF_NAME and start1 != sam.NO_REF_POS:
             raise ValueError("start1 cannot be used on its own - specify chrom or chrom1")
@@ -467,7 +412,7 @@ class SamBuilder:
         )
 
         # Sync up mate info and we're done!
-        self._set_mate_info(r1, r2)
+        sam.set_mate_info(r1, r2)
         self._records.append(r1)
         self._records.append(r2)
         return r1, r2
@@ -584,7 +529,7 @@ class SamBuilder:
                 file_handle = fp.file
 
             with sam.writer(
-                file_handle, header=self._samheader, file_type=sam.SamFileType.BAM  # type: ignore
+                file_handle, header=self._samheader, file_type=sam.SamFileType.BAM
             ) as writer:
                 for rec in self._records:
                     if pred(rec):
@@ -594,13 +539,11 @@ class SamBuilder:
 
             file_handle.close()
             if self._sort_order == SamOrder.QueryName:
-                # Ignore type hints for now until we have wrappers to use here.
-                pysam.sort("-n", *samtools_sort_args)  # type: ignore
+                pysam.sort("-n", *samtools_sort_args)
             elif self._sort_order == SamOrder.Coordinate:
-                # Ignore type hints for now until we have wrappers to use here.
                 if index:
                     samtools_sort_args.insert(0, "--write-index")
-                pysam.sort(*samtools_sort_args)  # type: ignore
+                pysam.sort(*samtools_sort_args)
 
         return path
 
