@@ -1,17 +1,15 @@
 """
-Methods for setting up logging for tools.
------------------------------------------
+# Methods for setting up logging for tools.
 
-Progress Logging Examples
-~~~~~~~~~~~~~~~~~~~~~~~~~
+## Progress Logging Examples
 
 Frequently input data (SAM/BAM/CRAM/VCF) are iterated in genomic coordinate order.  Logging
 progress is useful to not only log how many inputs have been consumed, but also their genomic
-coordinate.  :class:`~fgpyo.util.logging.ProgressLogger` can log progress every fixed number of
-records.  Logging can be written to :class:`logging.Logger` as well as custom print method.
-
-.. code-block:: python
+coordinate.  [`ProgressLogger()`][fgpyo.util.logging.ProgressLogger] can log progress every
+fixed number of records.  Logging can be written to `logging.Logger` as well as custom print
+method.
 
+```python
    >>> from fgpyo.util.logging import ProgressLogger
    >>> logged_lines = []
    >>> progress = ProgressLogger(
@@ -30,23 +28,18 @@ records.  Logging can be written to :class:`logging.Logger` as well as custom pr
    True
    >>> logged_lines  # show the lines logged
    ['recorded 2 items: chr1:2', 'recorded 3 items: chr1:3']
-
+```
 """
 
 import logging
-import sys
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
-
 import socket
 from contextlib import AbstractContextManager
 from logging import Logger
 from threading import RLock
 from typing import Any
 from typing import Callable
+from typing import Iterable
+from typing import Literal
 from typing import Optional
 from typing import Union
 
@@ -111,6 +104,7 @@ class ProgressLogger(AbstractContextManager):
         verb: str = "Read",
         unit: int = 100000,
     ) -> None:
+        self.printer: Callable[[str], Any]
         if isinstance(printer, Logger):
             self.printer = lambda s: printer.info(s)
         else:
@@ -170,6 +164,23 @@ class ProgressLogger(AbstractContextManager):
         else:
             return self.record(rec.reference_name, rec.reference_start + 1)
 
+    def record_alignments(
+        self,
+        recs: Iterable[AlignedSegment],
+    ) -> bool:
+        """Correctly record multiple pysam.AlignedSegments (zero-based coordinates).
+
+        Args:
+            recs: pysam.AlignedSegment objects
+
+        Returns:
+            true if a message was logged, false otherwise
+        """
+        logged_message: bool = False
+        for rec in recs:
+            logged_message = self.record_alignment(rec) or logged_message
+        return logged_message
+
     def _log(
         self,
         refname: Optional[str] = None,
@@ -193,6 +204,8 @@ class ProgressLogger(AbstractContextManager):
 
         self.printer(f"{self.verb} {self.count:,d} {self.noun}: {coordinate}")
 
+        return None
+
     def log_last(
         self,
     ) -> bool:
